From e887b2612a20380b534a9e3323c9a7ada57591bb Mon Sep 17 00:00:00 2001
From: Liang Zhang <zhangliang5@huawei.com>
Date: Mon, 1 Mar 2021 20:20:23 +0800
Subject: [PATCH 4/4] pidfile: clean pidfile when exit

Signed-off-by: Liang Zhang <zhangliang5@huawei.com>
---
 src/main.rs           | 15 ++++++++++++---
 util/src/daemonize.rs | 10 ++++++----
 2 files changed, 18 insertions(+), 7 deletions(-)

diff --git a/src/main.rs b/src/main.rs
index 5424994..ef2fed1 100644
--- a/src/main.rs
+++ b/src/main.rs
@@ -125,19 +125,28 @@ fn run() -> Result<()> {
 fn real_main(cmd_args: &arg_parser::ArgMatches, vm_config: VmConfig) -> Result<()> {
     let balloon_switch_on = vm_config.balloon.is_some();
 
+    TempCleaner::object_init();
+
     if cmd_args.is_present("daemonize") {
         match daemonize(cmd_args.value_of("pidfile")) {
-            Ok(()) => info!("Daemonize mode start!"),
-            Err(e) => error!("Daemonize start failed: {}", e),
+            Ok(()) => {
+                if let Some(pidfile) = cmd_args.value_of("pidfile") {
+                    TempCleaner::add_path(pidfile);
+                }
+                info!("Daemonize mode start!");
+            }
+            Err(e) => bail!("Daemonize start failed: {}", e),
         }
     } else {
+        if cmd_args.value_of("pidfile").is_some() {
+            bail!("-pidfile must be used with -daemonize together.");
+        }
         std::io::stdin()
             .lock()
             .set_raw_mode()
             .chain_err(|| "Failed to set terminal to raw mode.")?;
     }
 
-    TempCleaner::object_init();
     QmpChannel::object_init();
     EventLoop::object_init(&vm_config.iothreads)?;
     register_kill_signal();
diff --git a/util/src/daemonize.rs b/util/src/daemonize.rs
index dbb46b1..043251b 100644
--- a/util/src/daemonize.rs
+++ b/util/src/daemonize.rs
@@ -47,10 +47,6 @@ use crate::errors::{ErrorKind, Result};
 fn create_pid_file(path: &str) -> Result<()> {
     let pid: u32 = std::process::id();
 
-    if Path::new(path).exists() {
-        return Err(ErrorKind::PidFileExist.into());
-    }
-
     let mut pid_file: File = OpenOptions::new()
         .write(true)
         .create(true)
@@ -150,6 +146,12 @@ fn redirect_stdio(fd: RawFd) -> Result<()> {
 /// 4. Disassociate from the control terminal.
 /// 5. Write pid to pidfile.
 pub fn daemonize(pid_file: Option<String>) -> Result<()> {
+    if let Some(path) = pid_file.as_ref() {
+        if Path::new(path).exists() {
+            return Err(ErrorKind::PidFileExist.into());
+        }
+    }
+
     // The first fork make parent process quit, child process inherit parent's
     // session ID and have a new process ID. It can guarantee child
     // process will not be the first process in a session.
-- 
2.27.0

